@@ -6,7 +6,7 @@
 #define SHADOW_INCLUDE_LIB_FS_READLINK_READLINKNUL_H_
 
 
-#include <config.h>
+#include "config.h"
 
 #include <errno.h>
 #include <string.h>
@@ -17,7 +17,8 @@
 #include "sizeof.h"
 
 
-#define READLINKNUL(link, buf)  readlinknul(link, buf, countof(buf))
+// readlinknul_a - read link nul-terminate array
+#define readlinknul_a(link, buf)  readlinknul(link, buf, countof(buf))
 
 
 ATTR_STRING(1)
@@ -25,7 +26,7 @@ inline ssize_t readlinknul(const char *restrict link, char *restrict buf,
     ssize_t size);
 
 
-// Similar to readlink(2), but terminate the string.
+// readlinknul - read link nul-terminate
 inline ssize_t
 readlinknul(const char *restrict link, char *restrict buf, ssize_t size)
 {
