@@ -6,13 +6,14 @@
 // SPDX-License-Identifier: BSD-3-Clause
 
 
-#include <config.h>
+#include "config.h"
 
 #ident "$Id$"
 
 
 #include <ctype.h>
 #include <stdio.h>
+#include <string.h>
 
 #include "attr.h"
 #include "prototypes.h"
@@ -20,9 +21,9 @@
 #include "getdef.h"
 #include "string/ctype/strtoascii/strtolower.h"
 #include "string/memset/memzero.h"
-#include "string/sprintf/xaprintf.h"
+#include "string/sprintf/aprintf.h"
 #include "string/strcmp/streq.h"
-#include "string/strdup/xstrdup.h"
+#include "string/strdup/strdup.h"
 
 
 #if WITH_LIBBSD == 0
@@ -32,7 +33,8 @@
 /*
  * can't be a palindrome - like `R A D A R' or `M A D A M'
  */
-static bool palindrome (MAYBE_UNUSED const char *old, const char *new)
+static bool
+palindrome(const char *new)
 {
 	size_t i, j;
 
@@ -66,9 +68,8 @@ static bool similar (/*@notnull@*/const char *old, /*@notnull@*/const char *new)
 	}
 
 	for (i = j = 0; ('\0' != new[i]) && ('\0' != old[i]); i++) {
-		if (strchr (new, old[i]) != NULL) {
+		if (strchr(new, old[i]))
 			j++;
-		}
 	}
 
 	if (i >= j * 2) {
@@ -81,8 +82,7 @@ static bool similar (/*@notnull@*/const char *old, /*@notnull@*/const char *new)
 
 static /*@observer@*//*@null@*/const char *password_check (
 	/*@notnull@*/const char *old,
-	/*@notnull@*/const char *new,
-	/*@notnull@*/MAYBE_UNUSED const struct passwd *pwdp)
+	/*@notnull@*/const char *new)
 {
 	const char *msg = NULL;
 	char *oldmono, *newmono, *wrapped;
@@ -95,7 +95,7 @@ static /*@observer@*//*@null@*/const char *password_check (
 	oldmono = strtolower(xstrdup(old));
 	wrapped = xaprintf("%s%s", oldmono, oldmono);
 
-	if (palindrome (oldmono, newmono)) {
+	if (palindrome(newmono)) {
 		msg = _("a palindrome");
 	} else if (streq(oldmono, newmono)) {
 		msg = _("case changes only");
@@ -113,18 +113,19 @@ static /*@observer@*//*@null@*/const char *password_check (
 
 static /*@observer@*//*@null@*/const char *obscure_msg (
 	/*@notnull@*/const char *old,
-	/*@notnull@*/const char *new,
-	/*@notnull@*/const struct passwd *pwdp)
+	/*@notnull@*/const char *new)
 {
-	size_t maxlen, oldlen, newlen;
+	int maxlen, minlen;
+	size_t oldlen, newlen;
 	char *new1, *old1;
 	const char *msg;
-	const char *result;
 
 	oldlen = strlen (old);
 	newlen = strlen (new);
 
-	if (newlen < (size_t) getdef_num ("PASS_MIN_LEN", 0)) {
+	obscure_get_range(&minlen, &maxlen);
+
+	if (newlen < (size_t) minlen) {
 		return _("too short");
 	}
 
@@ -135,55 +136,34 @@ static /*@observer@*//*@null@*/const char *obscure_msg (
 		return NULL;
 	}
 
-	msg = password_check (old, new, pwdp);
+	msg = password_check(old, new);
 	if (NULL != msg) {
 		return msg;
 	}
 
-	result = getdef_str ("ENCRYPT_METHOD");
-	if (NULL == result) {
+	if (maxlen == -1) {
+		return NULL;
+	}
+
 	/* The traditional crypt() truncates passwords to 8 chars.  It is
 	   possible to circumvent the above checks by choosing an easy
 	   8-char password and adding some random characters to it...
 	   Example: "password$%^&*123".  So check it again, this time
 	   truncated to the maximum length.  Idea from npasswd.  --marekm */
 
-		if (getdef_bool ("MD5_CRYPT_ENAB")) {
-			return NULL;
-		}
-
-	} else {
-
-		if (   streq(result, "MD5")
-#ifdef USE_SHA_CRYPT
-		    || streq(result, "SHA256")
-		    || streq(result, "SHA512")
-#endif
-#ifdef USE_BCRYPT
-		    || streq(result, "BCRYPT")
-#endif
-#ifdef USE_YESCRYPT
-		    || streq(result, "YESCRYPT")
-#endif
-		    ) {
-			return NULL;
-		}
-
-	}
-	maxlen = getdef_num ("PASS_MAX_LEN", 8);
-	if (   (oldlen <= maxlen)
-	    && (newlen <= maxlen)) {
+	if (   (oldlen <= (size_t) maxlen)
+	    && (newlen <= (size_t) maxlen)) {
 		return NULL;
 	}
 
 	new1 = xstrdup (new);
 	old1 = xstrdup (old);
-	if (newlen > maxlen)
+	if (newlen > (size_t) maxlen)
 		stpcpy(&new1[maxlen], "");
-	if (oldlen > maxlen)
+	if (oldlen > (size_t) maxlen)
 		stpcpy(&old1[maxlen], "");
 
-	msg = password_check (old1, new1, pwdp);
+	msg = password_check(old1, new1);
 
 	freezero (new1, newlen);
 	freezero (old1, oldlen);
@@ -199,9 +179,10 @@ static /*@observer@*//*@null@*/const char *obscure_msg (
  *	check passwords.
  */
 
-bool obscure (const char *old, const char *new, const struct passwd *pwdp)
+bool
+obscure(const char *old, const char *new)
 {
-	const char *msg = obscure_msg (old, new, pwdp);
+	const char *msg = obscure_msg(old, new);
 
 	if (NULL != msg) {
 		printf (_("Bad password: %s.  "), msg);
@@ -209,3 +190,50 @@ bool obscure (const char *old, const char *new, const struct passwd *pwdp)
 	}
 	return true;
 }
+
+/*
+ * obscure_get_range - retrieve min and max password lengths
+ *
+ *  Returns minimum and maximum allowed lengths of a password
+ *  to pass obscure checks.
+ */
+void
+obscure_get_range(int *minlen, int *maxlen)
+{
+	int        val;
+	const char *method;
+
+	/* Minimum length is 0, even if -1 is configured. */
+	val = getdef_num("PASS_MIN_LEN", 0);
+	*minlen = val == -1 ? 0 : val;
+
+	/* Maximum password length check is optional. */
+	*maxlen = -1;
+
+	if (!getdef_bool("OBSCURE_CHECKS_ENAB")) {
+		return;
+	}
+
+	method = getdef_str ("ENCRYPT_METHOD");
+	if (NULL == method) {
+		if (getdef_bool ("MD5_CRYPT_ENAB")) {
+			return;
+		}
+	} else {
+		if (   streq(method, "MD5")
+#ifdef USE_SHA_CRYPT
+		    || streq(method, "SHA256")
+		    || streq(method, "SHA512")
+#endif
+#ifdef USE_BCRYPT
+		    || streq(method, "BCRYPT")
+#endif
+#ifdef USE_YESCRYPT
+		    || streq(method, "YESCRYPT")
+#endif
+		    ) {
+			return;
+		}
+	}
+	*maxlen = getdef_num ("PASS_MAX_LEN", 8);
+}
