@@ -12,7 +12,13 @@
 #ifndef _SGROUPIO_H
 #define _SGROUPIO_H
 
-extern int sgr_close (void);
+
+#include "config.h"
+
+#include "shadow/gshadow/sgrp.h"
+
+
+extern int sgr_close (bool process_selinux);
 extern bool sgr_file_present (void);
 extern /*@observer@*/ /*@null@*/const struct sgrp *sgr_locate (const char *name);
 extern int sgr_lock (void);
@@ -22,7 +28,7 @@ extern /*@null@*/const struct sgrp *sgr_next (void);
 extern int sgr_open (int mode);
 extern int sgr_remove (const char *name);
 extern int sgr_rewind (void);
-extern int sgr_unlock (void);
+extern int sgr_unlock (bool process_selinux);
 extern int sgr_update (const struct sgrp *sg);
 extern int sgr_sort (void);
 
