@@ -23,7 +23,7 @@
 ************************************************************************/
 
 #ifdef HAVE_CONFIG_H
-#include <config.h>
+#include "config.h"
 #endif
 
 #ifndef USE_PAM
@@ -100,7 +100,7 @@ login_access(const char *user, const char *from)
 	if (NULL != fp) {
 		intmax_t lineno = 0;	/* for diagnostics */
 		while (   !match
-		       && (fgets (line, sizeof (line), fp) == line))
+		       && (fgets(line, sizeof(line), fp) != NULL))
 		{
 			char  *p;
 
@@ -183,7 +183,7 @@ static char *myhostname (void)
 	static char name[MAXHOSTNAMELEN + 1] = "";
 
 	if (streq(name, "")) {
-		gethostname (name, sizeof (name));
+		gethostname(name, sizeof(name));
 		stpcpy(&name[MAXHOSTNAMELEN], "");
 	}
 	return (name);
@@ -315,9 +315,8 @@ static bool from_match (char *tok, const char *string)
 			return true;
 		}
 	} else if (strcaseeq(tok, "LOCAL")) {	/* LOCAL: no dots */
-		if (strchr (string, '.') == NULL) {
+		if (!strchr(string, '.'))
 			return true;
-		}
 	} else if (   (!streq(tok, "") && tok[strlen(tok) - 1] == '.') /* network */
 		   && strprefix(resolve_hostname(string), tok)) {
 		return true;
