@@ -4,7 +4,7 @@
  * SPDX-License-Identifier: BSD-3-Clause
  */
 
-#include <config.h>
+#include "config.h"
 #include <stdio.h>
 #include <string.h>
 #include <errno.h>
@@ -20,6 +20,7 @@
 #include "prototypes.h"
 #include "shadowlog.h"
 #include "string/strcmp/strprefix.h"
+#include "string/strerrno.h"
 #include "subordinateio.h"
 
 
@@ -124,7 +125,7 @@ int main(int argc, char **argv)
 	if (fstat(proc_dir_fd, &st) < 0) {
 		fprintf(stderr,
 		        _("%s: Could not stat directory for target process: %s\n"),
-		        Prog, strerror (errno));
+		        Prog, strerrno());
 		return EXIT_FAILURE;
 	}
 
@@ -143,10 +144,10 @@ int main(int argc, char **argv)
 		return EXIT_FAILURE;
 	}
 
-	if (!sub_uid_open(O_RDONLY)) {
+	if (want_subuid_file() && !sub_uid_open(O_RDONLY)) {
 		fprintf (stderr,
 		         _("%s: cannot open %s: %s\n"),
-		         Prog, sub_uid_dbname (), strerror (errno));
+		        Prog, sub_uid_dbname(), strerrno());
 		return EXIT_FAILURE;
 	}
 
@@ -158,7 +159,8 @@ int main(int argc, char **argv)
 	verify_ranges(pw, ranges, mappings);
 
 	write_mapping(proc_dir_fd, ranges, mappings, "uid_map", pw->pw_uid);
-	sub_uid_close();
+	if (want_subuid_file())
+		sub_uid_close(true);
 
 	return EXIT_SUCCESS;
 }
