@@ -30,7 +30,7 @@
  *
  */
 
-#include <config.h>
+#include "config.h"
 
 #ident "$Id$"
 
@@ -67,6 +67,11 @@
 #define E_MISSING	4	/* unexpected failure, passwd file missing */
 #define E_PWDBUSY	5	/* passwd file(s) busy */
 #define E_BADENTRY	6	/* bad shadow entry */
+
+struct option_flags {
+	bool chroot;
+};
+
 /*
  * Global variables
  */
@@ -76,14 +81,14 @@ static bool spw_locked = false;
 static bool pw_locked = false;
 
 /* local function prototypes */
-static void fail_exit (int status);
+static void fail_exit (int status, bool process_selinux);
 static void usage (int status);
-static void process_flags (int argc, char **argv);
+static void process_flags (int argc, char **argv, struct option_flags *flags);
 
-static void fail_exit (int status)
+static void fail_exit (int status, bool process_selinux)
 {
 	if (pw_locked) {
-		if (pw_unlock () == 0) {
+		if (pw_unlock (process_selinux) == 0) {
 			fprintf (stderr, _("%s: failed to unlock %s\n"), Prog, pw_dbname ());
 			SYSLOG ((LOG_ERR, "failed to unlock %s", pw_dbname ()));
 			/* continue */
@@ -91,7 +96,7 @@ static void fail_exit (int status)
 	}
 
 	if (spw_locked) {
-		if (spw_unlock () == 0) {
+		if (spw_unlock (process_selinux) == 0) {
 			fprintf (stderr, _("%s: failed to unlock %s\n"), Prog, spw_dbname ());
 			SYSLOG ((LOG_ERR, "failed to unlock %s", spw_dbname ()));
 			/* continue */
@@ -120,7 +125,7 @@ static void usage (int status)
  *
  *	It will not return if an error is encountered.
  */
-static void process_flags (int argc, char **argv)
+static void process_flags (int argc, char **argv, struct option_flags *flags)
 {
 	/*
 	 * Parse the command line options.
@@ -139,6 +144,7 @@ static void process_flags (int argc, char **argv)
 			usage (E_SUCCESS);
 			/*@notreached@*/break;
 		case 'R': /* no-op, handled in process_root_flag () */
+			flags->chroot = true;
 			break;
 		default:
 			usage (E_USAGE);
@@ -156,6 +162,8 @@ int main (int argc, char **argv)
 	struct passwd pwent;
 	const struct spwd *sp;
 	struct spwd spent;
+	struct option_flags  flags = {.chroot = false};
+	bool process_selinux;
 
 	log_set_progname(Prog);
 	log_set_logfd(stderr);
@@ -168,7 +176,8 @@ int main (int argc, char **argv)
 
 	OPENLOG (Prog);
 
-	process_flags (argc, argv);
+	process_flags (argc, argv, &flags);
+	process_selinux = !flags.chroot;
 
 #ifdef WITH_TCB
 	if (getdef_bool("USE_TCB")) {
@@ -181,33 +190,33 @@ int main (int argc, char **argv)
 		fprintf (stderr,
 		         _("%s: cannot lock %s; try again later.\n"),
 		         Prog, pw_dbname ());
-		fail_exit (E_PWDBUSY);
+		fail_exit (E_PWDBUSY, process_selinux);
 	}
 	pw_locked = true;
 	if (pw_open (O_CREAT | O_RDWR) == 0) {
 		fprintf (stderr,
 		         _("%s: cannot open %s\n"), Prog, pw_dbname ());
-		fail_exit (E_MISSING);
+		fail_exit (E_MISSING, process_selinux);
 	}
 
 	if (spw_lock () == 0) {
 		fprintf (stderr,
 		         _("%s: cannot lock %s; try again later.\n"),
 		         Prog, spw_dbname ());
-		fail_exit (E_PWDBUSY);
+		fail_exit (E_PWDBUSY, process_selinux);
 	}
 	spw_locked = true;
 	if (spw_open (O_CREAT | O_RDWR) == 0) {
 		fprintf (stderr,
 		         _("%s: cannot open %s\n"), Prog, spw_dbname ());
-		fail_exit (E_FAILURE);
+		fail_exit (E_FAILURE, process_selinux);
 	}
 
 	/*
 	 * Remove /etc/shadow entries for users not in /etc/passwd.
 	 */
 	(void) spw_rewind ();
-	while ((sp = spw_next ()) != NULL) {
+	while (NULL != (sp = spw_next())) {
 		if (pw_locate (sp->sp_namp) != NULL) {
 			continue;
 		}
@@ -219,7 +228,7 @@ int main (int argc, char **argv)
 			fprintf (stderr,
 			         _("%s: cannot remove entry '%s' from %s\n"),
 			         Prog, sp->sp_namp, spw_dbname ());
-			fail_exit (E_FAILURE);
+			fail_exit (E_FAILURE, process_selinux);
 		}
 		(void) spw_rewind();
 	}
@@ -229,7 +238,7 @@ int main (int argc, char **argv)
 	 * missing shadow entries.
 	 */
 	(void) pw_rewind ();
-	while ((pw = pw_next ()) != NULL) {
+	while (NULL != (pw = pw_next())) {
 		sp = spw_locate (pw->pw_name);
 		if (NULL != sp) {
 			/* do we need to update this entry? */
@@ -240,7 +249,7 @@ int main (int argc, char **argv)
 			spent = *sp;
 		} else {
 			/* add new shadow entry */
-			bzero(&spent, sizeof spent);
+			bzero(&spent, sizeof(spent));
 			spent.sp_namp   = pw->pw_name;
 			spent.sp_min    = getdef_num ("PASS_MIN_DAYS", -1);
 			spent.sp_max    = getdef_num ("PASS_MAX_DAYS", -1);
@@ -260,7 +269,7 @@ int main (int argc, char **argv)
 			fprintf (stderr,
 			         _("%s: failed to prepare the new %s entry '%s'\n"),
 			         Prog, spw_dbname (), spent.sp_namp);
-			fail_exit (E_FAILURE);
+			fail_exit (E_FAILURE, process_selinux);
 		}
 
 		/* remove password from /etc/passwd */
@@ -270,23 +279,23 @@ int main (int argc, char **argv)
 			fprintf (stderr,
 			         _("%s: failed to prepare the new %s entry '%s'\n"),
 			         Prog, pw_dbname (), pwent.pw_name);
-			fail_exit (E_FAILURE);
+			fail_exit (E_FAILURE, process_selinux);
 		}
 	}
 
-	if (spw_close () == 0) {
+	if (spw_close (process_selinux) == 0) {
 		fprintf (stderr,
 		         _("%s: failure while writing changes to %s\n"),
 		         Prog, spw_dbname ());
 		SYSLOG ((LOG_ERR, "failure while writing changes to %s", spw_dbname ()));
-		fail_exit (E_FAILURE);
+		fail_exit (E_FAILURE, process_selinux);
 	}
-	if (pw_close () == 0) {
+	if (pw_close (process_selinux) == 0) {
 		fprintf (stderr,
 		         _("%s: failure while writing changes to %s\n"),
 		         Prog, pw_dbname ());
 		SYSLOG ((LOG_ERR, "failure while writing changes to %s", pw_dbname ()));
-		fail_exit (E_FAILURE);
+		fail_exit (E_FAILURE, process_selinux);
 	}
 
 	/* /etc/passwd- (backup file) */
@@ -299,13 +308,13 @@ int main (int argc, char **argv)
 		/* continue */
 	}
 
-	if (pw_unlock () == 0) {
+	if (pw_unlock (process_selinux) == 0) {
 		fprintf (stderr, _("%s: failed to unlock %s\n"), Prog, pw_dbname ());
 		SYSLOG ((LOG_ERR, "failed to unlock %s", pw_dbname ()));
 		/* continue */
 	}
 
-	if (spw_unlock () == 0) {
+	if (spw_unlock (process_selinux) == 0) {
 		fprintf (stderr, _("%s: failed to unlock %s\n"), Prog, spw_dbname ());
 		SYSLOG ((LOG_ERR, "failed to unlock %s", spw_dbname ()));
 		/* continue */
