@@ -2,7 +2,7 @@
 // SPDX-License-Identifier: BSD-3-Clause
 
 
-#include <config.h>
+#include "config.h"
 
 #include <stdio.h>
 #include <string.h>
@@ -17,16 +17,16 @@
 #include "string/sprintf/snprintf.h"
 
 
-static void test_SNPRINTF_trunc(void **state);
-static void test_SNPRINTF_ok(void **state);
+static void test_stprintf_a_trunc(void **);
+static void test_stprintf_a_ok(void **);
 
 
 int
 main(void)
 {
     const struct CMUnitTest  tests[] = {
-        cmocka_unit_test(test_SNPRINTF_trunc),
-        cmocka_unit_test(test_SNPRINTF_ok),
+        cmocka_unit_test(test_stprintf_a_trunc),
+        cmocka_unit_test(test_stprintf_a_ok),
     };
 
     return cmocka_run_group_tests(tests, NULL, NULL);
@@ -34,33 +34,33 @@ main(void)
 
 
 static void
-test_SNPRINTF_trunc(void **state)
+test_stprintf_a_trunc(void **)
 {
 	char  buf[countof("foo")];
 
 	// Test that we're not returning SIZE_MAX
-	assert_true(SNPRINTF(buf, "f%su", "oo") < 0);
+	assert_true(stprintf_a(buf, "f%su", "oo") < 0);
 	assert_true(strcmp(buf, "foo") == 0);
 
-	assert_true(SNPRINTF(buf, "barbaz") == -1);
+	assert_true(stprintf_a(buf, "barbaz") == -1);
 	assert_true(strcmp(buf, "bar") == 0);
 }
 
 
 static void
-test_SNPRINTF_ok(void **state)
+test_stprintf_a_ok(void **)
 {
 	char  buf[countof("foo")];
 
-	assert_true(SNPRINTF(buf, "%s", "foo") == strlen("foo"));
+	assert_true(stprintf_a(buf, "%s", "foo") == strlen("foo"));
 	assert_true(strcmp(buf, "foo") == 0);
 
-	assert_true(SNPRINTF(buf, "%do", 1) == strlen("1o"));
+	assert_true(stprintf_a(buf, "%do", 1) == strlen("1o"));
 	assert_true(strcmp(buf, "1o") == 0);
 
-	assert_true(SNPRINTF(buf, "f") == strlen("f"));
+	assert_true(stprintf_a(buf, "f") == strlen("f"));
 	assert_true(strcmp(buf, "f") == 0);
 
-	assert_true(SNPRINTF(buf, "") == strlen(""));
+	assert_true(stprintf_a(buf, "") == strlen(""));
 	assert_true(strcmp(buf, "") == 0);
 }
